@@ -41,10 +41,14 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: cmd_go.c,v 1.1.1.1 2001/10/23 20:31:06 gray Exp $";
+static char RCS_Id[] = "$Id: cmd_go.c,v 1.4 2010/02/25 10:50:47 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
+/* sqsh-2.1.7 - New functions IgnoreCommentArgs and argv_shift */
+static int IgnoreCommentArgs _ANSI_ARGS(( int, char ** )) ;
+static void argv_shift _ANSI_ARGS(( int, char **, int )) ;
+
 /*
  * The following macro is used to convert a start time and end
  * time into total elapsed number of seconds, to the decimal
@@ -107,7 +111,17 @@ int cmd_go( argc, argv )
 	 */
 	env_tran( g_env );
 
-	while ((ch = sqsh_getopt( argc, argv, "nfhps:m:x;w:d:t;" )) != EOF) 
+	/*
+	 * sqsh-2.1.7 - Allow C style or ANSI SQL comments in \go command line.
+	 * Logically or physically move the commentary arguments to the end of
+	 * the argv list. Re-assign the effective number of arguments to argc.
+	 */
+	if ((argc = IgnoreCommentArgs ( argc, argv )) == -1)
+	{
+		fprintf( stderr, "\\go: Unbalanced comment tokens encountered\n" );
+		have_error = True;
+	}
+	else while ((ch = sqsh_getopt( argc, argv, "nfhps:m:x;w:d:t;T:" )) != EOF) 
 	{
 		switch (ch) 
 		{
@@ -180,6 +194,16 @@ int cmd_go( argc, argv )
 					have_error = True;
 				}
 				break;
+			case 'T' :
+				/*
+				 * sqsh-2.1.7 - Set a window title when using X result windows.
+				 */
+				if (env_put( g_env, "xwin_title", sqsh_optarg, ENV_F_TRAN ) == False)
+				{
+					fprintf( stderr, "\\go: -T: %s\n", sqsh_get_errstr() );
+					have_error = True;
+				}
+				break;
 
 			default :
 				fprintf( stderr, "\\go: %s\n", sqsh_get_errstr() );
@@ -193,19 +217,23 @@ int cmd_go( argc, argv )
 	 */
 	if( (argc - sqsh_optind) > 1 || have_error) 
 	{
-		fprintf( stderr, 
-			"Use: \\go [-d display] [-h] [-f] [-n] [-p] [-m mode] [-s sec]\n"
-			"          [-t [filter]] [-w width] [-x [xgeom]] [xacts]\n"
-			"     -d display When used with -x, send result to named display\n"
-			"     -h         Suppress headers\n"
-			"     -f         Suppress footers\n"
-			"     -n         Do not expand variables\n"
-			"     -m mode    Switch display mode for result set\n"
-			"     -s sec     Sleep sec seconds between transactions\n"
-			"     -t filter  Filter SQL through program\n"
-			"     -w width   Override value of $width\n"
-			"     -x xgeom   Override current value of $xgeom\n"
-			"     xacts      Repeat batch xacts times\n" );
+	    fprintf( stderr, 
+		"Use: \\go [-d display] [-h] [-f] [-n] [-p] [-m mode] [-s sec]\n"
+		"          [-t [filter]] [-w width] [-x [xgeom]] [-T title] [xacts]\n"
+		"     -d display  When used with -x, send result to named display\n"
+		"     -h          Suppress headers\n"
+		"     -f          Suppress footers\n"
+		"     -n          Do not expand variables\n"
+		"     -p          Report runtime statistics\n"
+		"     -m mode     Switch display mode for result set\n"
+		"     -s sec      Sleep sec seconds between transactions\n"
+		"     -t [filter] Filter SQL through program\n"
+		"                 Optional filter value overrides default variable $filter_prog\n"
+		"     -w width    Override value of $width\n"
+		"     -x [xgeom]  Send result set to a XWin output window\n"
+		"                 Optional xgeom value overrides default variable $xgeom\n"
+		"     -T title    Used in conjunction with -x to set window title\n"
+		"     xacts       Repeat batch xacts times\n" );
 
 		env_rollback( g_env );
 		return CMD_FAIL;
@@ -217,7 +245,7 @@ int cmd_go( argc, argv )
 	 */
 	if( argc != sqsh_optind ) 
 	{
-		iterations = atoi(argv[sqsh_optind]);
+		iterations = atoi(argv[argc-1]);
 
 		if( iterations < 1 ) 
 		{
@@ -544,7 +572,7 @@ cmd_go_interrupt:
 	if (!sqsh_stdin_isatty())
 		return_code = CMD_INTERRUPTED;
 	else
-		return_code = CMD_CLEARBUF;
+		return_code = CMD_RESETBUF;
 
 	goto cmd_go_leave;
 
@@ -554,13 +582,13 @@ cmd_go_error:
 	 * that the work buffer be cleared.
 	 */
 	if( clear_on_fail == NULL || *clear_on_fail == '1' )
-		return_code = CMD_CLEARBUF;
+		return_code = CMD_RESETBUF;
 	else
 		return_code = CMD_FAIL;
 	goto cmd_go_leave;
 
 cmd_go_succeed:
-	return_code = CMD_CLEARBUF;
+	return_code = CMD_RESETBUF;
 
 cmd_go_leave:
 	if (dsp_old != -1)
@@ -603,3 +631,89 @@ cmd_go_leave:
 
 	return return_code;
 }
+
+/*
+ * sqsh-2.1.7 - Function: IgnoreCommentArgs
+ *
+ * If the '\go' command line contains comments, then move the arguments that
+ * belong to a comment to the end of the argument list. The actual parameters
+ * are at the beginning of the list.
+ *
+ * Return the number of real arguments, that is the adjusted argc value.
+ * In case of error, return -1.
+ */
+int IgnoreCommentArgs (argc, argv)
+    int   argc;
+    char *argv[];
+{
+    int i = 1;  /* Skip command name argument argv[0] and start with argv[1] */
+
+    while ( i < argc )
+    {
+        if ( strcmp( argv[i], "--" ) == 0 )
+        {
+            /*
+             * Logically delete all remaining parameters.
+            */
+            return (i);
+        }
+        else if ( strcmp( argv[i], "/*" ) == 0 )
+        {
+            /*
+             * Move all the parameters in the C-style commentary block
+             * to the end of the argument list and decrement the
+             * count of valuable arguments. We do not have to increment 'i',
+             * because the argv_shift function shifts in the next argument to
+             * the current location.
+             * Note, because we decrement argc, the comment entries will
+             * eventually be stored in reverse order, but that's OK here.
+            */
+            do {
+                argv_shift ( argc--, argv, i );
+            } while ( i < argc && strcmp( argv[i], "*/" ) != 0 );
+
+            if ( i < argc && strcmp( argv[i], "*/" ) == 0 )
+                argv_shift ( argc--, argv, i );
+            else
+                return (-1); /* Missing end of comment token */
+        }
+        else if ( strcmp( argv[i], "*/" ) == 0 )
+            return (-1); /* Missing beginning of comment token */
+        else
+            i++;
+    }
+
+    return (argc);
+}
+
+/*
+ * sqsh-2.1.7 - Function: argv_shift
+ *
+ * Helper function to move an argument in an argv array to
+ * the end of the list and shift the rest of the arguments
+ * up to the named location.
+*/
+static void argv_shift( argc, argv, idx )
+    int   argc;
+    char  *argv[];
+    int   idx;
+{
+    char *cptr;
+    int   i;
+
+    /*-- Save pointer to argument --*/
+    cptr = argv[idx] ;
+
+    /*
+     * First, we need to shift the rest of the arguments down,
+     * removing this argument.
+    */
+    for( i = idx; i < argc - 1; i++ )
+        argv[i] = argv[i+1] ;
+
+    /*
+     * Now, put this argument to the end of the argv array.
+    */
+    argv[argc - 1] = cptr ;
+}
+
