@@ -31,7 +31,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: dsp.c,v 1.1.1.1 2001/10/23 20:31:06 gray Exp $";
+static char RCS_Id[] = "$Id: dsp.c,v 1.3 2008/05/21 17:51:24 mpeppler Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -73,7 +73,9 @@ dsp_prop_t  g_dsp_props = {
 	"|",             /* p_bcp_rowsep */
 	1,               /* p_bcp_rowsep_len */
 	1,               /* p_bcp_trim */
-	8192,            /* p_maxlen */
+	32768,            /* p_maxlen */
+	",",             /* p_csv_colsep */
+	1,               /* p_csv_colsep_len */
 };
 
 /*-- Prototypes --*/
@@ -217,6 +219,9 @@ int dsp_cmd( output, cmd, sql, flags )
 				case DSP_NONE:
 					dsp_func = dsp_none;
 					break;
+				case DSP_CSV:
+					dsp_func = dsp_csv;
+					break;
 				default:
 					dsp_func = dsp_html;
 			}
