@@ -36,7 +36,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: dsp_x.c,v 1.1.1.1 2001/10/23 20:31:06 gray Exp $";
+static char RCS_Id[] = "$Id: dsp_x.c,v 1.6 2010/02/08 13:25:48 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -49,7 +49,7 @@ int dsp_x( output, cmd, flags, dsp_func )
 	dsp_t       *dsp_func;
 {
 	fprintf( stderr, "dsp_x: X11 support not compiled into sqsh\n" );
-	return DSP_SUCCEED;
+	return DSP_FAIL;
 }
 
 #else
@@ -92,8 +92,6 @@ int dsp_x( output, cmd, flags, dsp_func )
 {
 	int    fd_pair[2];               /* Pair of pipes to communicate */
 	int    ret;                      /* Return value from display method */
-	int    argc;                     /* Required by XtInitialize */
-	char  *argv[1];                  /* Required by XtInitialize */
 	int    width;                    /* Width of screen */
 	int    height;                   /* Height of screen */
 	char   number[20];               /* Holder for number */
@@ -270,26 +268,56 @@ static int dsp_x_init( fd, width, height )
 	XmString     s_cancel;
 	cb_data_t    cd;                 /* Data for callbacks */
 	XtInputId    id;                 /* Id of input source */
+	Dimension    dim;
 	int          text_width;
 	int          orig_width;
 	int          argc;
 	char        *argv[1];
 	char        *cp;
+	char        *xwin_title = NULL;
+	varbuf_t    *exp_buf;
 
 	/*
 	 * At this point we are in the child process, the rest is pretty
 	 * easy.  We simply open a window and simply place every line we
 	 * receive from the parent through the fd file descriptor
 	 * into a window.
+	 * sqsh-2.1.7 - Pass on a window title through argv[0].
 	 */
-	argc    = 0;
-	argv[0] = NULL;
+	env_get( g_env, "xwin_title", &xwin_title );
+	if (xwin_title != NULL && *xwin_title != '\0')
+	{
+		exp_buf = varbuf_create( 64 );
+
+		if (exp_buf == NULL)
+		{
+			fprintf( stderr, "sqsh: %s\n", sqsh_get_errstr() );
+			sqsh_exit( 255 );
+		}
+
+		if (sqsh_expand( xwin_title, exp_buf, 0 ) == False)
+		{
+			fprintf( stderr, "sqsh: Error expanding $xwin_title: %s\n",
+				sqsh_get_errstr() );
+			sqsh_exit( 255 );
+		}
+		argc    = 1;
+		argv[0] = varbuf_getstr( exp_buf );
+	}
+	else
+	{
+		argc    = 0;
+		argv[0] = NULL;
+	}
 
 	/*
 	 * Calculate the number of lines in the SQL Text buffer.
+	 * sqsh-2.1.7 : Set a maximum of 10 lines for the SQL buffer,
+	 * otherwise it might eat up all the X window space in case of a
+	 * large SQL batch.
 	 */
 	nlines = 0;
-	for (cp = varbuf_getstr(g_sqlbuf); *cp != '\0'; ++cp)
+	for (cp = varbuf_getstr(g_sqlbuf); *cp != '\0' && nlines < 10; ++cp)
 	{
 		if (*cp == '\n')
 		{
@@ -378,8 +406,20 @@ static int dsp_x_init( fd, width, height )
 	cd.c_id      = id;
 
 	XtRealizeWidget( w_top );
+ 	/*
+ 	 * Ok, We want to prevent the button from being resized. The easiest
+ 	 * way to accomplish that is to set the max and the min.
+ 	 */
+ 	XtVaGetValues( w_btn_form, 
+		      XmNheight,       &dim, 
+ 		      NULL);
+	XtVaSetValues( w_btn_form, 
+ 		      XmNpaneMaximum,  dim, 
+ 		      XmNpaneMinimum,  dim, 
+ 		      NULL);
 	XtMainLoop();
 
+	varbuf_destroy( exp_buf );
 	exit(0);
 }
 
@@ -481,8 +521,10 @@ static int dsp_x_init( fd, width, height )
    cb_data_t    cd;                 /* Data for callbacks */
    XtInputId    id;                 /* Id of input source */
    int          text_width;
-	int          argc;
-	char        *argv[1];
+   int          argc;
+   char        *argv[1];
+   char        *xwin_title = NULL;
+   varbuf_t    *exp_buf;
 
    XFontStruct  *font = NULL; /* Font for text widget */
 
@@ -491,9 +533,33 @@ static int dsp_x_init( fd, width, height )
 	 * easy.  We simply open a window and simply place every line we
 	 * receive from the parent through the fd file descriptor
 	 * into a window.
+	 * sqsh-2.1.7 - Pass on a window title through argv[0].
 	 */
-	argc    = 0;
-	argv[0] = NULL;
+	env_get( g_env, "xwin_title", &xwin_title );
+	if (xwin_title != NULL && *xwin_title != '\0')
+	{
+		exp_buf = varbuf_create( 64 );
+
+		if (exp_buf == NULL)
+		{
+			fprintf( stderr, "sqsh: %s\n", sqsh_get_errstr() );
+			sqsh_exit( 255 );
+		}
+
+		if (sqsh_expand( xwin_title, exp_buf, 0 ) == False)
+		{
+			fprintf( stderr, "sqsh: Error expanding $xwin_title: %s\n",
+				sqsh_get_errstr() );
+			sqsh_exit( 255 );
+		}
+		argc    = 1;
+		argv[0] = varbuf_getstr( exp_buf );
+	}
+	else
+	{
+		argc    = 0;
+		argv[0] = NULL;
+	}
 
 	/*-- Intialize our X Session --*/
 	w_top = XtInitialize( "sqsh", "Sqsh", NULL, 0, &argc, argv );
@@ -509,8 +575,8 @@ static int dsp_x_init( fd, width, height )
 		XtNright,             XtChainRight,
 		XtNwidth,             200,
 		XtNheight,            100,
-		XtNscrollHorizontal,  XawtextScrollWhenNeeded,
-		XtNscrollVertical,    XawtextScrollWhenNeeded,
+		XtNscrollHorizontal,  XawtextScrollAlways,
+		XtNscrollVertical,    XawtextScrollAlways,
 		XtNeditType,          XawtextAppend,
 		NULL );
 
@@ -574,6 +640,7 @@ static int dsp_x_init( fd, width, height )
 	XtRealizeWidget( w_top );
 	XtMainLoop();
 
+	varbuf_destroy( exp_buf );
 	exit(0);
 }
 
@@ -671,3 +738,4 @@ static void dsp_x_input_cb (client_data, fd, id)
 
 #endif /* !USE_MOTIF */
 #endif /* USE_X11 */
+
