@@ -42,7 +42,7 @@
 
 /*-- Current Version --*/
 #if !defined(lint) && !defined(__LINT__)
-static char RCS_Id[] = "$Id: sqsh_job.c,v 1.1.1.1 2001/10/23 20:31:06 gray Exp $";
+static char RCS_Id[] = "$Id: sqsh_job.c,v 1.4 2010/01/28 15:30:37 mwesdorp Exp $";
 USE(RCS_Id)
 #endif /* !defined(lint) */
 
@@ -113,10 +113,11 @@ jobset_t* jobset_create( hsize )
 	 * Allocate a sigcld_t.  This will act as a handle on all of the
 	 * SIGCHLD events that will be received due to background jobs
 	 * completing.
+	 * sqsh-2.1.7 - Logical fix: Free js->js_jobs before a free of js.
 	 */
 	if( (js->js_sigcld = sigcld_create()) == NULL ) {
-		free( js );
 		free( js->js_jobs );
+		free( js );
 		sqsh_set_error( sqsh_get_error(), "sigcld_create: %s",
 		                sqsh_get_errstr() );
 	 	return NULL;
@@ -407,22 +408,27 @@ job_id_t jobset_run( js, cmd_line, exit_status )
 				 * We want the child to process signals a little differently
 				 * from the parent.  There is no real need for graceful re-
 				 * covery within the child.
+				 * sqsh-2.1.7 - Ignore SIGINT (Ctrl-C) interrupts from parent.
+				 *              Let the child perform a proper sqsh_exit().
+				 *              Return code is not very relevant here.
 				 */
 				while (sig_restore() >= 0);
-					
+				sig_install ( SIGINT, SIG_H_IGN, NULL, 0 );
 				ret = cmd->cmd_ptr( args_argc(job->job_args), 
 					args_argv(job->job_args) );
-				exit(ret);
+				sqsh_exit(0);
 
 			/*
 			 * Parent process. All we need to do is record the pid of the
 			 * child in the job structure, restore the  file descriptors
 			 * and return.
+			 * sqsh-2.1.7 - Make sure SIGCHLD signals are unblocked.
 			 */
 			default :  /* Parent */
 				
 				job->job_pid = child_pid;
 				sigcld_watch( js->js_sigcld, job->job_pid );
+				sigcld_unblock () ;
 		}
 
 		/*
@@ -594,7 +600,8 @@ job_id_t jobset_wait( js, job_id, exit_status, block_type )
 		return jobset_wait_all( js, exit_status, block_type );
 
 	/*-- Find out if the job exists --*/
-	for( j = js->js_jobs[js->js_hsize % job_id];
+	/* (sqsh-2.1.7 - Bug fix for bucket calculation) --*/
+	for( j = js->js_jobs[job_id % js->js_hsize];
 		  j != NULL && j->job_id != job_id ; j = j->job_nxt );
 
 	/*-- If we can't, error --*/
@@ -623,15 +630,6 @@ job_id_t jobset_wait( js, job_id, exit_status, block_type )
 	 */
 	pid = sigcld_wait( js->js_sigcld, j->job_pid, exit_status, wait_type );
 
-	/*
-	 * Propogate any errors back to the caller.
-	 */
-	if( pid == -1 ) {
-		sqsh_set_error( sqsh_get_error(), "sigcld_wait: %s",
-							 sqsh_get_errstr() );
-		return -1;
-	}
-
 	/*
 	 * If block_type was JOB_NONBLOCK and there were no jobs pending,
 	 * then return 0.
@@ -649,7 +647,21 @@ job_id_t jobset_wait( js, job_id, exit_status, block_type )
 	j->job_end     = time(NULL);
  	j->job_status  = *exit_status;
 
- 	sqsh_set_error( SQSH_E_NONE, NULL );
+	/*
+	 * Propagate any errors back to the caller.
+	 * sqsh-2.1.7 - If we missed a SIGCLD signal and the pid is already
+	 * finished, then remove the pid from the watch list, report an error
+	 * and continue as normal. Then you can use \show to check the deferred
+	 * output so far and get the job out of the queue.
+	 */
+	if( pid == -1 ) {
+		sigcld_unwatch ( js->js_sigcld, j->job_pid );
+		sqsh_set_error( sqsh_get_error(), "sigcld_wait: %s", sqsh_get_errstr() );
+	}
+	else {
+	 	sqsh_set_error( SQSH_E_NONE, NULL );
+	}
+
  	return j->job_id;
 }
 
@@ -662,7 +674,7 @@ int jobset_end( js, job_id )
 	int         exit_status;
 
 	/*-- Always check your parameters --*/
-	if( js == NULL ) {
+	if( js == NULL || job_id <= 0 ) {
 		sqsh_set_error( SQSH_E_BADPARAM, NULL );
 		return False;
 	}
@@ -904,7 +916,7 @@ static job_id_t jobset_wait_all( js, exit_status, block_type )
 	pid = sigcld_wait( js->js_sigcld, -1, exit_status, wait_type );
 
 	/*
-	 * Propogate any errors back to the caller.
+	 * Propagate any errors back to the caller.
 	 */
 	if( pid == -1 ) {
 		sqsh_set_error( sqsh_get_error(), "sigcld_wait: %s", sqsh_get_errstr() );
@@ -981,6 +993,9 @@ static int jobset_parse( js, job, cmd_line, while_buf, tok_flags )
 	char              *bg_ptr   = NULL ; /* Location of & */
 	char              *pipe_ptr = NULL ; /* Location of | */
 	char              *cp;
+	/* sqsh-2.1.6 - New variables */
+	varbuf_t	  *exp_buf;
+
 
 	/*
 	 * In a "real" shell both the | pipeline(s) and the & background
@@ -1037,10 +1052,25 @@ static int jobset_parse( js, job, cmd_line, while_buf, tok_flags )
 
 			if( tmp_dir == NULL || *tmp_dir == '\0' )
 				tmp_dir = SQSH_TMP;
+			else /* sqsh-2.1.6 feature - Expand tmp_dir variable */
+		       	{
+				if ((exp_buf = varbuf_create( 512 )) == NULL)
+				{
+					sqsh_set_error( sqsh_get_error(),"varbuf: %s",
+							sqsh_get_errstr() );
+					return False;
+				}
+				if (sqsh_expand( tmp_dir, exp_buf, 0 ) == False)
+					tmp_dir = SQSH_TMP;
+				else
+					tmp_dir = varbuf_getstr( exp_buf );
+			}
 
 			/*-- Create the defer file --*/
 			sprintf( defer_path, "%s/sqsh-dfr.%d-%d", 
 			         tmp_dir, (int) getpid(), job->job_id );
+
+			varbuf_destroy( exp_buf ); /* sqsh-2.1.6 feature */
 			
 			/*-- Let the job structure know where it is --*/
 			if( (job->job_output = sqsh_strdup( defer_path )) == NULL ) {
@@ -1049,7 +1079,7 @@ static int jobset_parse( js, job, cmd_line, while_buf, tok_flags )
 			}
 
 			/*-- Now, open the file --*/
-			if( (defer_fd = sqsh_open(defer_path,O_CREAT|O_WRONLY,0600)) == -1 ) {
+			if( (defer_fd = sqsh_open(defer_path,O_CREAT|O_WRONLY|O_TRUNC,0600)) == -1 ) {
 				sqsh_set_error( sqsh_get_error(), "Unable to open %s: %s", 
 				                defer_path, sqsh_get_errstr() );
 				return False;
